# -*- coding: utf-8 -*-
# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from datetime import date

from medlemssys.medlem.behaviours import Lookup
from medlemssys.medlem.behaviours import MedlemQuerySet
from medlemssys.medlem.models import Medlem

from .base import BaseBehaviour


class BarnOgUngdomMedlemQuerySet(MedlemQuerySet):
    ung_alder = 25

    def unge(self, year=date.today().year):
        """Medlem rekna som unge (altso i teljande alder)"""
        return self.ikkje_utmelde(year) \
            .filter(
                fodt__gte = year - self.ung_alder
            )

    def potensielt_teljande(self, year=date.today().year):
        return self.unge(year).filter(postnr__gt="0000", postnr__lt="9999") \
            .exclude(giroar__gjeldande_aar=year,
                giroar__innbetalt__isnull=False)

    def teljande(self, year=date.today().year):
        """Medlem i teljande alder, med postnr i Noreg og med betalte
        medlemspengar"""
        return self.betalande(year) & self.unge(year).distinct().filter(postnr__gt="0000", postnr__lt="9999")

    def teljande_nye(self, year=date.today().year):
        """Nyinnmelde medlem som er teljande"""
        return self.nye(year) & self.teljande(year)


class Behaviour(BaseBehaviour):
    queryset_class = BarnOgUngdomMedlemQuerySet
    medlem_ui_filters = BaseBehaviour.medlem_ui_filters + [
        Lookup('teljande', "Teljande",
               lambda qs: qs.teljande()),
        Lookup('potensielle', "Potensielt teljande",
               lambda qs: qs.potensielt_teljande()),
        Lookup('nye_teljande', "Teljande (nye i år)",
               lambda qs: qs.teljande(date.today().year - 1)),
        Lookup('teljandeifjor', "Teljande (i fjor)",
               lambda qs: qs.teljande(date.today().year - 1)),
    ]
    giro_ui_filters = BaseBehaviour.giro_ui_filters + [
        Lookup('teljandeifjor', "Teljande (i fjor)",
               lambda qs: (
                   qs.filter(medlem__in=(Medlem.objects.teljande()
                                         .values_list('pk', flat=True)))
                   .filter(gjeldande_aar=date.today().year))),
        Lookup('teljande', "Teljande (i år)",
               lambda qs: (
                   qs.filter(medlem__in=(Medlem.objects.teljande(
                                             date.today().year - 1)
                                         .values_list('pk', flat=True)))
                   .filter(gjeldande_aar=date.today().year - 1))),
        Lookup('potensieltteljande', "Potensielt teljande (i år)",
               lambda qs: (
                   qs.filter(medlem__in=(Medlem.objects.potensielt_teljande()
                                         .values_list('pk', flat=True)))
                   .filter(gjeldande_aar=date.today().year))),
    ]



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import json

from reversion import revisions as reversion
from django.contrib.auth.decorators import login_required
from django.db.models import Count
from django.db.models import Q
from django.http import HttpResponse
from django.http import HttpResponseForbidden
from django.http import HttpResponseRedirect
from django.shortcuts import render

from models import EndraMedlemForm
from models import InnmeldingMedlemForm
from models import Lokallag
from models import Medlem
from models import Val


def create_medlem(request):
    if request.method == 'POST':
        form = InnmeldingMedlemForm(request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect('/takk/')
    else:
        form = InnmeldingMedlemForm()

    return render(request, 'medlem/create.html', {
        'form': form,
    })


def edit_medlem(request, id, nykel):
    forbid = False
    try:
        m = Medlem.objects.get(pk=id)
    except Medlem.DoesNotExist:
        forbid = True
    if forbid or nykel != m.nykel:
        return HttpResponseForbidden("<h1>Medlemen finst ikkje, "
                                     "eller nykelen er feil</h1>")
    if request.method == 'POST':
        form = EndraMedlemForm(request.POST, instance=m)
        post_val = request.POST.get('val_epost', '')
        post_val += request.POST.get('val_post', '')
        post_val += request.POST.get('val_tlf',  '')
        post_val += request.POST.get('val_nyhendebrev',  '')
        if form.is_valid():
            with reversion.create_revision():
                added = []
                removed = []
                for tittel in ('Ikkje epost', u'Ikkje Motmæle', 'Ikkje ring',
                               'Ikkje Norsk Tidend', 'Ikkje nyhendebrev'):
                               #'Ikkje lokallagsepost', 'Ikkje SMS', 'Ikkje nyhendebrev'):
                    val_obj = Val.objects.get(tittel=tittel)
                    if tittel in post_val:
                        if not m.val_exists(tittel):
                            added.append(tittel)
                        m.val.add(val_obj)
                    else:
                        if m.val_exists(tittel):
                            removed.append(tittel)
                        m.val.remove(val_obj)
                comment = "Brukar oppdaterte {0}.".format(form.changed_data)
                if added:
                    comment += " La til: {0}.".format(added)
                if removed:
                    comment += " Tok vekk: {0}.".format(removed)
                reversion.set_comment(comment)
                form.save()
    else:
        form = EndraMedlemForm(instance=m)

    return render(request, 'medlem/edit.html', {
       'object': m,
       'form': form,
    })


@login_required
def ringjelister(request):
    lokallag = Lokallag.objects.all().order_by('andsvar')

    return render(request, 'medlem/ringjeliste.html', {
        'lokallag': lokallag,
    })


@login_required
def search(request):
    return render(request, 'medlem/search.html', {})


def get_members(terms, limit=20):
    search = None

    # Søk etter mobil- eller medlemsnummer
    if "".join(terms).isdigit():
        q = "".join(terms)
        search = Q(mobnr__startswith=q) | Q(id__startswith=q)
        terms = []

    # Vanleg søk
    for q in terms:
        search_part = (
            Q(fornamn__istartswith=q) |
            Q(mellomnamn__istartswith=q) |
            Q(etternamn__istartswith=q) |
            Q(fodt__icontains=q) |
            Q(lokallag__namn__icontains=q) |
            Q(lokallag__slug__istartswith=q))
        if not search:
            search = search_part
        else:
            search = search & search_part

    results = (Medlem.objects.select_related('lokallag')
               .prefetch_related('giroar')
               .annotate(null_last=Count('_siste_medlemspengar'))
               .order_by('-null_last', '-_siste_medlemspengar', '-fodt', 'pk'))
    if search:
        results = results.filter(search)
    return results[:limit]


@login_required
def get_members_json(request):
    term = request.GET.get('term', '')
    results = []
    for member in get_members(term.split()):
        member_json = {}
        member_json['id'] = member.pk
        member_json['namn'] = str(member)
        member_json['alder'] = member.alder()
        if member.lokallag_id:
            member_json['lokallag'] = str(member.lokallag)
        else:
            member_json['lokallag'] = "(ingen)"
        bet = (giro.gjeldande_aar for giro in
               member.giroar.order_by('-gjeldande_aar') if
               giro.innbetalt)
        member_json['bet'] = [unicode(x) for x in bet]
        results.append(member_json)
    data = json.dumps(results)
    mimetype = 'application/json'
    return HttpResponse(data, mimetype)



# -*- coding: utf-8 -*-
# vim: shiftwidth=4 tabstop=4 expandtab softtabstop=4 ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import print_function

import csv
import os

from django.core.management.base import BaseCommand
from django.core.management.base import CommandError

from medlemssys.medlem.models import PostNummer

obj = ""

class Command(BaseCommand):
    args = '<Tab seperated Erik Bolstad postnr-CSV files>'
    help = """Importerer medlemane inn i databasen"""

    def handle(self, *args, **options):
        global obj
        obj = self
        if 'force_update' in options:
            self.force_update = True

        if not os.path.isfile(args[0]):
            raise CommandError("Fila finst ikkje ({0})".format(args[0]).encode('utf8'))

        # 0       1         2                3            4         5      6       7        8      9    10   11            12                       13
        # POSTNR, POSTSTAD, POSTNR- OG STAD, BRUKSOMRÅDE, FOLKETAL, BYDEL, KOMMNR, KOMMUNE, FYLKE, LAT, LON, DATAKVALITET, DATAKVALITETSFORKLARING, SIST OPPDATERT
        csv.register_dialect('tabs', delimiter='\t')
        read = csv.reader(open(args[0]), dialect='tabs')
        row = read.next()
        if row[0] != 'POSTNR' or row[11] != 'DATAKVALITET':
            raise CommandError("Ser ikkje ut som den korrekte type fila")

        for row in read:
            p = PostNummer()
            p.postnr = row[0].strip().replace(' ', '')
            p.poststad = row[1]
            p.bruksomrade = row[3]
            if (row[4] != ""):
                p.folketal = int(row[4].strip())
            p.bydel = row[5]
            p.kommnr = row[6]
            p.kommune = row[7]
            p.fylke = row[8]
            p.lat = float(row[9])
            p.lon = float(row[10])
            p.datakvalitet = int(row[11])
            if row[-1][0] == "2":
                p.sist_oppdatert = row[-1]

            p.save()
#           print "'%s' '%s' '%s'" % (row, row[6:7], row[7:8])



def stderr(msg):
    if obj:
        obj.stderr.write((unicode(msg) + "\n").encode('utf-8'))
    else:
        print((unicode(msg)).encode('utf-8'))




# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-30 21:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='GiroTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subject', models.CharField(max_length=100)),
                ('text_template', models.TextField()),
                ('html_template', models.TextField()),
                ('trigger', models.CharField(max_length=100, unique=True)),
            ],
        ),
    ]



from django.conf.urls import patterns, url

urlpatterns = patterns('aarsmelding.views',
    url(r'^$',
        'index', name='aarsmelding_home'),
    url(r'^(?P<lokallag_slug>[-\w]+)/$',
        'single', name='aarsmelding_single'),
)



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from collections import namedtuple
from datetime import date

from django.db.models import F
from django.db.models import Q
from django.db.models.query import QuerySet

from .models import PostNummer


Lookup = namedtuple('Lookup', 'key title filter')

MEDLEM_LOOKUPS = [
    Lookup('ikkje_utmelde', "Ikkje utmelde",
           lambda qs: qs.ikkje_utmelde()),
    Lookup('interessante', "Interessante",
           lambda qs: qs.interessante()),
    Lookup('betalande', "Betalande",
           lambda qs: qs.betalande()),
    Lookup('betalandeifjor', "Betalande (i fjor)",
           lambda qs: qs.betalande(date.today().year - 1)),
    Lookup('nye', "Nye i år",
           lambda qs: qs.nye()),
    Lookup('nye_betalande', "Nye i år (betalande)",
           lambda qs: qs.betalande_nye()),
]

def strange_giro(queryset):
    not_fully_paid = (
        ~Q(innbetalt_belop=0) & ~Q(innbetalt_belop=F('belop')))
    paid_but_not_finished = Q(innbetalt__isnull=False) & ~Q(status='F')
    unpaid_but_finished = (
        Q(innbetalt__isnull=True) | Q(innbetalt_belop=0)
            & Q(status='F'))
    return queryset.filter(not_fully_paid |
                           paid_but_not_finished |
                           unpaid_but_finished)

GIRO_LOOKUPS = [
    Lookup('strange', "Rare/feil giroar", strange_giro),
]


class MedlemQuerySet(QuerySet):
    """Inherit from this base, call the class MedlemQuerySet and
    refer to it from BEHAVIOURS_MODULE in settings"""

    def alle(self):
        """Alle oppføringar i registeret"""
        if getattr(self, 'core_filters', None):
            return self.filter(**self.core_filters)
        return self

    def ikkje_utmelde(self, year=date.today().year):
        """Medlem som ikkje er eksplisitt utmelde"""
        return self.alle().filter(
            Q(utmeldt_dato__isnull=True) | Q(utmeldt_dato__gte=date(year+1, 1, 1))
        )

    def utmelde(self, year=date.today().year):
        """Medlem som er utmelde"""
        return self.alle().filter(
            utmeldt_dato__isnull=False, utmeldt_dato__lt=date(year+1, 1, 1)
        )

    def betalande(self, year=date.today().year):
        """Medlem med ein medlemspengeinnbetaling inneverande år"""
        return self.ikkje_utmelde(year) \
            .filter(
                giroar__gjeldande_aar=year,
                giroar__innbetalt__isnull=False
            ).distinct()

    def nye(self, year=date.today().year):
        """Nyinnmelde medlem i dette året"""
        return self.ikkje_utmelde(year).filter(innmeldt_dato__year=year).distinct()

    def betalande_nye(self, year=date.today().year):
        """Nyinnmelde medlem som har betalt"""
        return self.nye(year) & self.betalande(year)

    def interessante(self, year=date.today().year):
        """Medlem som har betalt i år eller i fjor."""
        return self.ikkje_utmelde(year) \
            .filter(
                Q(innmeldt_dato__year=year) |
                Q(giroar__gjeldande_aar=year,
                  giroar__innbetalt__isnull=False) |
                Q(giroar__gjeldande_aar=year-1,
                  giroar__innbetalt__isnull=False)
            ).distinct()

    # Postnummer
    def fylke(self, fylke):
        """Medlem som ikkje er eksplisitt utmelde"""
        nr = PostNummer.objects.filter(fylke=fylke).values_list('postnr', flat=True)
        # Converting the QuerySet __in to a list because of a MySQL performance issue
        return self.alle().filter(
                   postnr__in=list(nr.values_list('postnr', flat=True).distinct()))

    def kommune(self, kommune, fylke=None):
        """Medlem som ikkje er eksplisitt utmelde"""
        nr = PostNummer.objects.filter(kommune=kommune)
        if fylke:
            nr = nr.filter(fylke=fylke)
        # Converting the QuerySet __in to a list because of a MySQL performance issue
        return self.alle().filter(
                   postnr__in=list(nr.values_list('postnr', flat=True).distinct()))



# Create your views here.
import datetime
from django.shortcuts import render, get_object_or_404
from medlem.models import Lokallag

def index(request):
    return render(request, 'aarsmelding/index.html', {})

def single(request, lokallag_slug):
    lokallag = get_object_or_404(Lokallag, slug=lokallag_slug)
    year = datetime.date.today().year - 1
    previous_year = year - 1

    teljande = lokallag.medlem_set.teljande(year)
    previous_teljande = lokallag.medlem_set.teljande(previous_year)

    betalande = lokallag.medlem_set.betalande(year)
    previous_betalande = lokallag.medlem_set.betalande(previous_year)

    nye = lokallag.medlem_set.nye(year)
    previous_nye = lokallag.medlem_set.nye(previous_year)

    teljande_nye = lokallag.medlem_set.teljande_nye(year)
    previous_teljande_nye = lokallag.medlem_set.teljande_nye(previous_year)
    return render(request, 'aarsmelding/single.html', {
        'lokallag': lokallag,
        'teljande': teljande,
        'previous_teljande': previous_teljande,
        'betalande': betalande,
        'previous_betalande': previous_betalande,
        'nye': nye,
        'previous_nye': previous_nye,
        'teljande_nye': teljande_nye,
        'previous_teljande_nye': previous_teljande_nye,
        'year': year,
        'previous_year': previous_year,
    })



# -*- coding: utf-8 -*-
# vim: ts=4 sw=4 sts=4 expandtab ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from django.contrib.auth.decorators import login_required
from django.contrib.auth.views import redirect_to_login
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Count
from django.shortcuts import render_to_response, get_object_or_404

from models import Medlem, Lokallag


@login_required
def home(request):
    lokallag = Lokallag.objects.annotate(num_medlemar=Count('medlem'))  \
                               .filter(num_medlemar__gt=0)              \
                               .order_by("-num_medlemar")

    return render_to_response('lokallag/lokallag_listing.html', {
        'lokallag': lokallag,
    })


def lokallag_home(request, slug):
    lokallag = get_object_or_404(Lokallag, slug=slug)

    if not request.user.is_staff:
        if request.user.is_authenticated():
            try:
                medlem = request.user.medlem
            except ObjectDoesNotExist:
                return render_to_response(
                    "error.html",
                    {
                        'message': "Brukaren din er ikkje kopla opp mot ein "
                        "medlemsprofil. Det må til for å få løyve til å sjå "
                        "ting."
                    })
        elif request.GET.get('user_id'):
            medlem = get_object_or_404(Medlem, pk=request.GET['user_id'])
            if (not medlem.nykel or medlem.nykel != request.GET.get('nykel')):
                return redirect_to_login(request.get_full_path())
        else:
            return redirect_to_login(request.get_full_path())
        if not medlem.lokallagsrolle.filter(slug=slug).exists():
            return render_to_response(
                "error.html",
                {'message': "Du har inga rolle i dette lokallaget."})

    medlem = Medlem.objects.interessante().filter(lokallag=lokallag)
    betalt_count = Medlem.objects.betalande().filter(lokallag=lokallag).count()
    teljande_count = Medlem.objects.teljande().filter(lokallag=lokallag).count()

    return render_to_response('lokallag/lokallag_home.html', {
        'lokallag': lokallag,
        'medlem': medlem,
        'betalt_count': betalt_count,
        'teljande_count': teljande_count,
    })




from __future__ import absolute_import

import os

from celery import Celery
from django.apps import AppConfig
from django.conf import settings


if not settings.configured:
    # set the default Django settings module for the 'celery' program.
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'config.settings.local')  # pragma: no cover


app = Celery('medlemssys')


class CeleryConfig(AppConfig):
    name = 'medlemssys.taskapp'
    verbose_name = 'Celery Config'

    def ready(self):
        # Using a string here means the worker will not have to
        # pickle the object when using Windows.
        app.config_from_object('django.conf:settings')
        app.autodiscover_tasks(lambda: settings.INSTALLED_APPS, force=True)

        if hasattr(settings, 'RAVEN_CONFIG'):
            # Celery signal registration
            from raven import Client as RavenClient
            from raven.contrib.celery import register_signal as raven_register_signal
            from raven.contrib.celery import register_logger_signal as raven_register_logger_signal

            raven_client = RavenClient(dsn=settings.RAVEN_CONFIG['DSN'])
            raven_register_logger_signal(raven_client)
            raven_register_signal(raven_client)


@app.task(bind=True)
def debug_task(self):
    print('Request: {0!r}'.format(self.request))  # pragma: no cover



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from medlemssys.medlem.behaviours import MedlemQuerySet
from medlemssys.medlem.behaviours import MEDLEM_LOOKUPS
from medlemssys.medlem.behaviours import GIRO_LOOKUPS


class BaseBehaviour(object):
    queryset_class = MedlemQuerySet
    medlem_ui_filters = MEDLEM_LOOKUPS
    giro_ui_filters = GIRO_LOOKUPS


class Behaviour(BaseBehaviour):
    pass



from reportlab.lib.enums import TA_JUSTIFY
from reportlab.lib.pagesizes import a4
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Image
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.units import inch

def giro(request, queryset):
    doc = SimpleDocTemplate("form_letter.pdf",pagesize=a4,
                                 rightMargin=72,leftMargin=72,
                                 topMargin=72,bottomMargin=18)
    Story=[]
    #im = Image(logo, 2*inch, 2*inch)
    #Story.append(im)
    styles=getSampleStyleSheet()
    styles.add(ParagraphStyle(name='Justify', alignment=TA_JUSTIFY))

    ptext = '<font size=16>%s</font>' % request.POST.get('title').strip()
    Story.append(Paragraph(ptext, styles["Normal"]))
    Story.append(Spacer(1, 12))

    for medlem in queryset:
        LocalStory = Story
        ptext = '<font size=12>Dear %s:</font>' % medlem
        LocalStory.append(Paragraph(ptext, styles["Normal"]))
        LocalStory.append(Spacer(1, 12))

        ptext = '<font size=12>%s</font>' % request.POST.get('text')
        LocalStory.append(Paragraph(ptext, styles["Justify"]))
        LocalStory.append(Spacer(1, 12))

        # lag side


def ringjelister_pdf(request):
    from cStringIO import StringIO
    #from reportlab.pdfgen import canvas
    #from reportlab.lib.units import cm #, mm
    from reportlab.lib.styles import getSampleStyleSheet
    from reportlab.platypus import *
    from reportlab.lib import colors

    response = HttpResponse(mimetype="application/pdf")
    response['Content-Disposition'] = 'filename=noko.pdf'
    # Our container for 'Flowable' objects
    elements = []

    # A large collection of style sheets pre-made for us
    styles = getSampleStyleSheet()

    # A basic document for us to write to 'rl_hello_table.pdf'
    buf = StringIO()
    doc = SimpleDocTemplate(buf)

    elements.append(Paragraph("Wumpus vs Cave Population Report",
     styles['Title']))

    data2 = Medlem.objects.all()[100]
    data = [['Caves',         'Wumpus Population'],
            ['Deep Ditch',    50],
            ['Death Gully',   5000],
            ['Dire Straits',  600],
            ['Deadly Pit',    5],
            ['Conclusion',    'Run!']]

    # First the top row, with all the text centered and in Times-Bold,
    # and one line above, one line below.
    ts = [('ALIGN', (1,1), (-1,-1), 'CENTER'),
         ('LINEABOVE', (0,0), (-1,0), 1, colors.purple),
         ('LINEBELOW', (0,0), (-1,0), 1, colors.purple),
         ('FONT', (0,0), (-1,0), 'Times-Bold'),

    # The bottom row has one line above, and three lines below of
    # various colors and spacing.
         ('LINEABOVE', (0,-1), (-1,-1), 1, colors.purple),
         ('LINEBELOW', (0,-1), (-1,-1), 0.5, colors.purple,
          1, None, None, 4,1),
         ('LINEBELOW', (0,-1), (-1,-1), 1, colors.red),
         ('FONT', (0,-1), (-1,-1), 'Times-Bold')]

    # Create the table with the necessary style, and add it to the
    # elements list.
    table = Table(data2, style=ts)
    elements.append(table)

    # Write the document to disk
    doc.build(elements)
    pdf = buf.getvalue()
    response.write(pdf)
    return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Medlemssys
==========

Medlemssys is a membership register.  Primarily written for norwegian youth
organizations, but can and is used by other kinds of organizations.

It doesn't do anything very advanced, but provides a way to list the members,
export to CSV, can read OCR-files, send emails and some more.

Installation
------------

Doing a simple install, you can install "medlemssys" for a test version, or
"medlemssys[production]" to get a few required extras for a real setup::

    pip install medlemssys[production]

You'll use the `medlemssys` command for all commands.  It's a small wrapper
around the django-admin command.  The first thing you'll want to do is to
create a settings file::

    medlemssys init

This will give you a `medlemssys_conf.py` file.  Have a look at it, and change
what you need.  You should load in the data to the database and create yourself
a user::

    medlemssys migrate
    medlemssys createsuperuser

If you chose the production setup, you can run gunicorn::

    export DJANGO_SETTINGS_MODULE=medlemssys_conf
    gunicorn --bind 0.0.0.0:8000 medlemssys.config.wsgi:application

You can of course also try the django development server to just test your
register::

    medlemssys runserver 0.0.0.0:8000

"""

from setuptools import setup
from setuptools import find_packages


setup(
    name='medlemssys',
    use_scm_version=True,
    description='Membership register for norwegian youth organizations',
    long_description=__doc__,
    license='AGPLv3+',
    author='Odin Hørthe Omdal',
    author_email='odin.omdal@gmail.com',
    url='https://github.com/odinho/medlemssys',
    install_requires=[
        'Django >= 1.7, < 1.10',
        'logan',
        'whitenoise',
        'django-model-utils',
        'pytz',
        'reportlab',
        'python-dateutil',
        'django-reversion >= 2.0',
        'django-reversion-compare >= 0.7',
    ],
    extras_require={
        'production': [
            'django-gunicorn',
            'gunicorn',
            'psycopg2',
            'raven',
        ],
    },
    setup_requires=['setuptools_scm'],
    entry_points={
        'console_scripts': [
            'medlemssys = medlemssys.runner:main',
        ],
    },
    packages=find_packages(),
    package_data={
        '': ['templates/*', 'templates/**/*', 'templates/**/**/*',
             'static/**', 'static/**/*'],
        'medlemssys.giro': ['OCRB.ttf'],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Database :: Front-Ends',
        'Topic :: Office/Business',
    ],
)



# -*- coding: utf-8 -*-
# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from django.shortcuts import render
from reversion.models import Revision

def show_revisions(request):
    SHOW_CHANGES = 50
    revision_list = []
    for r in Revision.objects.all().order_by('-date_created')[:250]:
        if revision_list and str(revision_list[-1][0]) == str(r):
            revision_list[-1][0].comment += "\n" + r.comment
            continue
        str_ver = []
        for version in r.version_set.all()[:SHOW_CHANGES]:
            try:
                str_ver.append(version.object.admin_change())
            except (AttributeError, ValueError):
                str_ver.append(unicode(version))
        total_changes = r.version_set.count()
        text = u', '.join(str_ver)
        if total_changes > SHOW_CHANGES:
            text = u"Totalt {0} endringar: {1} ...".format(total_changes, text)
        revision_list.append((r, text))

    return render(request, 'admin/revision_list.html', {
        'revision_list': revision_list,
        'title': 'Versjonar',
    })



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-30 23:00
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('medlem', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='LokallagStat',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('veke', models.CharField(max_length=7, verbose_name='\xc5r-veke')),
                ('n_teljande', models.IntegerField(verbose_name='tal p\xe5 teljande')),
                ('n_interessante', models.IntegerField(verbose_name='tal p\xe5 interessante')),
                ('n_ikkje_utmelde', models.IntegerField(verbose_name='tal p\xe5 ikkje utmelde')),
                ('n_totalt', models.IntegerField(verbose_name='tal p\xe5 medlemar registrert')),
                ('teljande', models.TextField(verbose_name='liste over teljande')),
                ('interessante', models.TextField(verbose_name='liste over interessante')),
                ('oppretta', models.DateTimeField(auto_now_add=True, verbose_name='oppretta')),
                ('lokallag', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='medlem.Lokallag')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='lokallagstat',
            unique_together=set([('lokallag', 'veke')]),
        ),
    ]



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import datetime

from django.contrib import admin
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from reversion_compare.admin import CompareVersionAdmin

from . import admin_actions
from . import filters
from . import models


class GiroInline(admin.TabularInline):
    model = models.Giro
    extra = 1
    classes = ['left']
    fields = ['belop', 'innbetalt_belop', 'kid', 'gjeldande_aar', 'innbetalt', 'konto', 'desc', 'status']
    readonly_fields = ('status',)
class RolleInline(admin.TabularInline):
    model = models.Rolle
    extra = 1
    classes = ['left']
class VervaMedlemInline(admin.TabularInline):
    model = models.Medlem
    fk_name = 'verva_av'
    verbose_name = _("verving")
    verbose_name_plural = _("vervingar")
    extra = 0
    classes = ['left']
    fields = [
        'admin_change', 'innmeldingstype', 'innmeldingsdetalj',
        'lokallag', 'postnr', 'innmeldt_dato', '_siste_medlemspengar',
        'fodt_farga', 'er_innmeldt', 'har_betalt']
    readonly_fields = (
        'innmeldingstype', 'admin_change', 'postnr', 'innmeldt_dato',
        '_siste_medlemspengar', 'fodt_farga', 'er_innmeldt', 'har_betalt')


class MedlemAdmin(CompareVersionAdmin):
    list_display = ('id', '__unicode__', 'lokallag_changelist', 'er_innmeldt',
                    'siste_giro_info', 'fodt_farga', 'status_html')
    list_display_links = ('id', '__unicode__')
    date_hierarchy = 'innmeldt_dato'
    list_filter = (
            filters.SporjingFilter,
            ('val', filters.AdditiveSubtractiveFilter),
            #('_siste_medlemspengar', TimeSinceFilter),
            filters.FodtFilter,
            filters.StadFilter,
            '_siste_medlemspengar',
            'innmeldt_dato',
            'utmeldt_dato',
            'status',
            'lokallag',
            'giroar__status',
        )
    raw_id_fields = ['verva_av', 'betalt_av']
    readonly_fields = ('_siste_medlemspengar', 'oppretta', 'oppdatert')
    save_on_top = True
    inlines = [RolleInline, GiroInline, VervaMedlemInline]
    search_fields = ('fornamn', 'mellomnamn', 'etternamn', '=id', '^mobnr',)
    filter_horizontal = ('val',)
    fieldsets = (
        (None, {
            'classes': ('left',),
            'fields': (
                ('fornamn', 'mellomnamn', 'etternamn', 'fodt', 'kjon'),
                ('postadr', 'postnr', 'ekstraadr'),
                ('mobnr', 'epost'),
                ('lokallag', 'status', 'innmeldt_dato'),
            )
            }),
        (u'Ikkje pakravde felt', {
            'classes': ('left', 'collapse'),
            'fields': (
                ('utmeldt_dato', '_siste_medlemspengar', 'user', 'nykel'),
                ('heimenr', 'gjer'),
                ('innmeldingstype', 'innmeldingsdetalj', 'verva_av'),
                'merknad',
                ('borteadr', 'bortepostnr', 'betalt_av'),
                ('val',),
                ('oppretta', 'oppdatert'),
            )
        }),
    )
    actions = [ admin_actions.simple_member_list,
                admin_actions.csv_list,
                admin_actions.lag_giroar,
                admin_actions.suggest_lokallag,
                admin_actions.members_meld_ut,
              ]

    def get_actions(self, request):
        """Puts the delete action on the bottom"""
        actions = super(MedlemAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            deleteaction = actions['delete_selected']
            del actions['delete_selected']
            actions['delete_selected'] = deleteaction
        return actions


    def get_queryset(self, request):
        self._get_params = request.GET

        # use our manager, rather than the default one
        qs = self.model.objects.alle()

        # we need this from the superclass method
        ordering = self.ordering or () # otherwise we might try to *None, which is bad ;)
        if ordering:
            qs = qs.order_by(*ordering)
        return qs

    def lokallag_changelist(self, obj):
        url = reverse('admin:medlem_medlem_changelist')
        querystring = self._get_params.copy()
        if not obj.lokallag:
            querystring['lokallag__isnull'] = True
        else:
            querystring['lokallag__id__exact'] = obj.lokallag.pk
        return u'<a href="{0}?{1}">{2}</a>'.format(url,
                                                   querystring.urlencode(),
                                                   obj.lokallag_display())
    lokallag_changelist.short_description = _("Lokallag")
    lokallag_changelist.admin_order_field = 'lokallag__namn'
    lokallag_changelist.allow_tags = True

    def siste_giro_info(self, obj):
        gjeldande = obj.gjeldande_giro()
        if gjeldande:
            return gjeldande.admin_change()
        fjor = obj.gjeldande_giro(datetime.date.today().year - 1)
        if fjor:
            return '{0}: {1}'.format(fjor.gjeldande_aar, fjor.admin_change())
        return '&mdash;'
    siste_giro_info.short_description = _("Siste giro")
    siste_giro_info.admin_order_field = 'giroar'
    siste_giro_info.allow_tags = True


class RolleInline(admin.TabularInline):
    model = models.Rolle
    extra = 0
    raw_id_fields = ['medlem']


class LokallagAdmin(CompareVersionAdmin):
    list_display = ('pk', 'namn_med_stad', 'num_medlem', 'andsvar',
                    'epost', 'aktivt', 'styret_admin', 'overvakingar')
    list_editable = ('andsvar', 'aktivt',)
    list_per_page = 200
    list_filter = (
        'aktivt',
        'rolle_set__rolletype',
        'lokalsats',
    )
    inlines = [RolleInline,]
    prepopulated_fields = {'slug': ('namn',)}

    def overvakingar(self, obj):
        def html(o):
            return u'<a href="{url}" title="Til: {epostar}">{tekst}</a>'.format(
                url=reverse('admin:medlem_lokallagovervaking_change',
                            args=(o.pk,)),
                epostar=o.epostar_admin(), tekst=unicode(o))
        return u', '.join(html(o) for o in obj.lokallag_overvaking_set.all())
    overvakingar.allow_tags = True

    def namn_med_stad(self, obj):
        return u'<span title="kommunar: {}\nfylke: {}">{}</span>'.format(
            obj.kommunes, obj.fylke, obj.namn)
    namn_med_stad.allow_tags = True
    namn_med_stad.admin_order_field = 'namn'


class LokallagOvervakingAdmin(CompareVersionAdmin):
    date_hierarchy = 'sist_oppdatert'
    model = models.LokallagOvervaking
    list_display = ('__unicode__', 'lokallag', 'medlem', 'epost',
                    'styreepostar', 'sist_oppdatert')
    list_per_page = 250
    list_filter = ('deaktivert', 'lokallag__aktivt')
    fields = ('medlem', 'epost', 'lokallag', 'deaktivert', 'sist_oppdatert',
              'epostar_admin')
    readonly_fields = ('sist_oppdatert', 'epostar_admin',)
    raw_id_fields = ['medlem']

    def styreepostar(self, obj):
        return ', '.join(obj.lokallag.styreepostar())


class ValInline(admin.TabularInline):
    model = models.Medlem.val.through
    raw_id_fields = ['medlem']
class ValAdmin(CompareVersionAdmin):
    list_display = ('tittel', 'forklaring', 'num_medlem')
    model = models.Val
    inlines = [ValInline,]
    fieldsets = (
        (None, {
            'fields': ('tittel', 'forklaring'),
            'description': "Val er avslått som standard, so det er lurt å utforma "
                           "vala slik at dei fleste ikkje treng ha dei på. T.d. "
                           "«Ikkje ring» i staden for «Ring».",
        }),
    )


class GiroAdmin(CompareVersionAdmin):
    model = models.Giro
    raw_id_fields = ['medlem']
    list_display = ('pk', 'medlem_admin_change', 'kid', 'belop',
                    'innbetalt_belop', 'gjeldande_aar', 'innbetalt', 'konto',
                    'status')
    list_editable = ('innbetalt', 'innbetalt_belop', 'gjeldande_aar', 'status', )
    date_hierarchy = 'oppretta'
    search_fields = ('=id', '=kid', 'medlem__pk', 'medlem__fornamn',
                     'medlem__etternamn',)
    list_filter = (
        'status',
        filters.GiroSporjingFilter,
        'gjeldande_aar',
        'innbetalt',
        'hensikt',
        'konto',
        'belop',
        filters.MedlemFodtFilter,
        'medlem__lokallag',
        'medlem__status',
        'medlem__utmeldt_dato',
        'medlem__innmeldt_dato',
    )
    readonly_fields = ('oppretta',)
    fieldsets = (
        (None, {
            'fields': (
                'medlem',
                ('belop', 'innbetalt_belop'),
                ('konto', 'hensikt'),
                'innbetalt',
                'kid',
                ('gjeldande_aar', 'oppretta', 'status'),
                'desc',
            )
        }),
    )
    actions = [
                admin_actions.giro_status_ferdig,
                admin_actions.giro_status_postlagt,
                admin_actions.giro_status_ventar,
                admin_actions.giro_list,
                admin_actions.csv_list,
                admin_actions.pdf_giro,
            ]

    def medlem_admin_change(self, obj):
        return obj.medlem.admin_change()
    medlem_admin_change.short_description = _("Medlem")
    medlem_admin_change.admin_order_field = 'medlem'
    medlem_admin_change.allow_tags = True


class RolleAdmin(CompareVersionAdmin):
    list_filter = ('rolletype', 'lokallag')
    list_display = ('id', 'medlem_admin_change', 'lokallag', 'rolletype')
    raw_id_fields = ('medlem', 'lokallag')

    def medlem_admin_change(self, obj):
        return obj.medlem.admin_change()
    medlem_admin_change.short_description = _("Medlem")
    medlem_admin_change.admin_order_field = 'medlem'
    medlem_admin_change.allow_tags = True


class RolletypeAdmin(CompareVersionAdmin):
    list_display = ('namn', 'rolle_num')
    inlines = [RolleInline,]


class PostNummerAdmin(CompareVersionAdmin):
    list_display = ('__unicode__', 'postnr', 'poststad',
                    'num_teljande_medlem', 'num_medlem', 'folketal',
                    'bydel', 'kommune', 'fylke', 'sist_oppdatert')
    list_filter = ('fylke', 'datakvalitet')
    date_hierarchy = 'sist_oppdatert'
    list_per_page = 400


admin.site.register(models.Medlem, MedlemAdmin)
admin.site.register(models.Lokallag, LokallagAdmin)
admin.site.register(models.Val, ValAdmin)
admin.site.register(models.Giro, GiroAdmin)
admin.site.register(models.LokallagOvervaking, LokallagOvervakingAdmin)
admin.site.register(models.Rolletype, RolletypeAdmin)
admin.site.register(models.Rolle, RolleAdmin)
admin.site.register(models.PostNummer, PostNummerAdmin)
